@@ -4,8 +4,14 @@
 #
 
 
-fitfclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
-                             fpcCtrl, p=2, pert=0.01){
+fitfclustWrapper <- function(data, 
+														 k,
+														 reg,
+														 regTime,
+														 funcyCtrlMbc,
+                             fpcCtrl,
+														 p=2,
+														 pert=0.01){
     ##input
     dimBase <- funcyCtrlMbc@dimBase
     baseType <- funcyCtrlMbc@baseType
@@ -32,13 +38,25 @@ fitfclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 
     ##evaluation
     ptm <- proc.time()
-    res <- fitfclust(data=data, dimBase=dimBase, h=redDim, p=p, k=k,
-                     regTime=regTime,
-                     epsilon=epsilon, maxiter=maxiter, pert=pert,
-                     hard=hard, seed=seed, init=init, nrep=nrep, reg=reg,
-                     fpcCtrl=fpcCtrl, baseType=baseType)
-    sysTime <- proc.time()-ptm
     
+    res <- fitfclust(data=data, 
+    								 dimBase=dimBase, 
+    								 h=redDim, 
+    								 p=p, 
+    								 k=k,
+                     regTime=regTime,
+                     epsilon=epsilon, 
+    								 maxiter=maxiter, 
+    								 pert=pert,
+                     hard=hard, 
+    								 seed=seed, 
+    								 init=init, 
+    								 nrep=nrep, 
+    								 reg=reg,
+                     fpcCtrl=fpcCtrl, 
+    								 baseType=baseType)
+    
+    sysTime <- proc.time()-ptm
 
     prms <- res$parameters
     ##class prediction
@@ -74,13 +92,25 @@ fitfclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 }
 
 
-distclustWrapper <- function(data, k, reg, regTime, funcyCtrl,
-                            fpcCtrl, method="pam")
+distclustWrapper <- function(data, 
+														 k,
+														 reg,
+														 regTime,
+														 funcyCtrl,
+														 fpcCtrl,
+														 method="pam")
     {
         ##evaluation
         ptm <- proc.time()
-        res <- distclust(data, k, reg, regTime, funcyCtrl,
-                        fpcCtrl, method=method)
+        
+        res <- distclust(data, 
+        								 k,
+        								 reg,
+        								 regTime,
+        								 funcyCtrl,
+        								 fpcCtrl,
+        								 method=method)
+        
         sysTime <- proc.time()-ptm
 
         ##funcyOut
@@ -101,14 +131,23 @@ distclustWrapper <- function(data, k, reg, regTime, funcyCtrl,
     }
 
 
-iterSubspaceWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
-                            fpcCtrl, simplif=TRUE)
+iterSubspaceWrapper <- function(data, 
+																k,
+																reg,
+																regTime,
+																funcyCtrlMbc,
+																fpcCtrl,
+																simplif=TRUE)
     {
         ##evaluation
         ptm <- proc.time()
-        res <- iterSubspace(data=data, k=k, regTime=regTime,
-                        reg=reg, funcyCtrlMbc=funcyCtrlMbc,
-                        fpcCtrl=fpcCtrl, simplif=simplif)
+        res <- iterSubspace(data=data, 
+        										k=k, 
+        										regTime=regTime,
+        										reg=reg, 
+        										funcyCtrlMbc=funcyCtrlMbc,
+        										fpcCtrl=fpcCtrl, 
+        										simplif=simplif)
         sysTime <- proc.time()-ptm
         
         ##funcyOut
@@ -135,12 +174,14 @@ iterSubspaceWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 }
 
 
-funclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
-                            nbInit=5, nbIterInit=50, ...){
+funclustWrapper <- function(data, 
+														k,
+														reg,
+														regTime,
+														funcyCtrlMbc,
+                            nbInit=5,
+														nbIterInit=50, ...){
 
-    if(!requireNamespace("Funclustering"))
-        stop("Please install Funclustering to use this method.")
-    
     if(!reg)
         stop("This method does not work on sparse data!")
     baseType <- funcyCtrlMbc@baseType
@@ -160,13 +201,23 @@ funclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 
     ##evaluation
     ptm <- proc.time()
+    
     res <- formatFuncy(data, regTime=regTime,  format="Format3")
     data <- t(res$Yin); t_all <- res$t_all
     baseObj <- makeBasis(baseType, t_all, nbasis=dimBase)$bObj
     fd <- Data2fd(data, argvals=t_all, basisobj=baseObj);
-    res=Funclustering::funclust(fd=fd, K=k, thd=thd, increaseDimension=increaseDimension, hard=hard, fixedDimension=fixedDimension, nbInit=nbInit,
-        nbIterInit=nbIterInit, nbIteration=nbIteration,
-        epsilon=epsilon, ...)
+    res <- 
+    	Funclustering::funclust(fd=fd, 
+    													K=k, 
+    													thd=thd, 
+    													increaseDimension=increaseDimension, 
+    													hard=hard, 
+    													fixedDimension=fixedDimension, 
+    													nbInit=nbInit,
+    													nbIterInit=nbIterInit, 
+    													nbIteration=nbIteration,
+    													epsilon=epsilon, ...)
+    
     sysTime <- proc.time()-ptm
 
     clout <- label2lowerk(res$cls)
@@ -198,11 +249,13 @@ funclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 }
 
 
-funHDDCWrapper <- function(data, k, reg, regTime,  funcyCtrlMbc,
+funHDDCWrapper <- function(data, 
+													 k,
+													 reg,
+													 regTime,
+													 funcyCtrlMbc,
                            model="AkBkQkDk", ...){
 
-     if(!requireNamespace("funHDDC"))
-        stop("Please install funHDDC to use this method.")
      if(!reg)
         stop("This method does not work on sparse data!")
     baseType=funcyCtrlMbc@baseType
@@ -220,21 +273,45 @@ funHDDCWrapper <- function(data, k, reg, regTime,  funcyCtrlMbc,
     seed <- funcyCtrlMbc@seed
 
     ptm <- proc.time()
-    res <- formatFuncy(data, regTime=regTime, format="Format3")
+    
+    res <- formatFuncy(data, 
+    									 regTime=regTime, 
+    									 format="Format3")
+    
     data <- t(res$Yin); t_all <- res$t_all
     baseObj <- makeBasis(baseType, t_all, nbasis=dimBase)$bObj
     fd <- Data2fd(data, argvals=t_all, basisobj=baseObj);
     set.seed(seed)
-    res=try(funHDDC::funHDDC(fd=fd, K=k, init=init, model=model, thd=thd,
-        maxit=maxit ,eps=eps, ...), silent=TRUE)
-    if(class(res)=="try-error"){
-        warning(paste("Clustering with", k,
-                      "classes is not possible.", k-1,
-                      "clusters are used!"))
-        k <- k-1
-        return(funHDDCWrapper(data, k, reg, regTime, funcyCtrlMbc,
-                              model="AkBkQkDk", ...))
+
+    # recursive wrapper function to automatically decrease the number of classes
+    # in case funHDDC() raises any error or the warning "All models diverged."
+    .funHDDC_auto <- function(data, K, init, model, threshold, itermax, eps, ...)
+    {
+      args <- c(list(data = data, K = K, init = init, model = model, 
+                     threshold = threshold, itermax = itermax, eps = eps),
+                ...)
+      
+      withRestarts(
+        tryCatch(do.call(funHDDC::funHDDC, args), 
+                 error = function(e) invokeRestart("reduceClusterCount"),
+                 warning = function(w) {
+                   if (w$message == "All models diverged.") {
+                     warning(paste("Clustering with", args$K, 
+                                   "classes is not possible.",	args$K - 1, 
+                                   "clusters are used."), call. = FALSE)
+                     invokeRestart("reduceClusterCount")
+                   }
+                 }),
+        reduceClusterCount = function() {
+          args$K <- args$K - 1
+          do.call(.funHDDC_auto, args)
+        }
+      )
     }
+    
+    res <- .funHDDC_auto(data = fd, K = k, init = init, model = model, 
+                         threshold = thd, itermax = maxit, eps = eps, ...)
+    
     sysTime <- proc.time()-ptm
 
     ##funcyOut
@@ -242,21 +319,21 @@ funHDDCWrapper <- function(data, k, reg, regTime,  funcyCtrlMbc,
     out@methodName <- "funHDDC"
     out@kOut <- k
     out@time <- t_all
-    out@dimBaseOut <- max(res$cls)
-    out@cluster <- res$cls
-    fdmeans <- fd; fdmeans$coefs <- t(res$prms$m)
+    out@dimBaseOut <- max(res$class)
+    out@cluster <- res$class
+    fdmeans <- fd; fdmeans$coefs <- t(res$mu)
     out@centers <- eval.fd(t_all, fdmeans)
-    out@props <- round(as.numeric(unlist(res$prms$prop)),4)
+    out@props <- round(as.numeric(res$prop),4)
     out@dist2centers <- dist2centers(data, out@centers)
     out@cldist=makeClMat(out@dist2centers)
     out@calcTime<- sysTime 
     ##funcyOutMbc 
-    out@groupDimBase <- as.numeric(res$prms$d)
-    out@probs<-res$P
-    out@prms <- res$prms
-    out@AIC <- -res$aic
-    out@BIC <- -2*res$bic
-    out@logLik <- res$loglik[length(res$loglik)]
+    out@groupDimBase <- as.numeric(res$d)
+    out@probs<-res$posterior
+    out@prms <- list(a = res$a, b = res$b)
+    out@AIC <- as.integer(NA)
+    out@BIC <- res$BIC
+    out@logLik <- res$loglik_all
     out@nrIter <- as.integer(NA)
     
     return(out)
@@ -264,16 +341,34 @@ funHDDCWrapper <- function(data, k, reg, regTime,  funcyCtrlMbc,
 
 
 
-fscmWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
-                        fpcCtrl, location=NULL, scale=FALSE,
-                        knn=5, useCode="C", verbose=FALSE){
+fscmWrapper <- function(data, 
+												k, 
+												reg,
+												regTime,
+												funcyCtrlMbc,
+                        fpcCtrl, 
+												location=NULL,
+												scale=FALSE,
+                        knn=5, 
+												useCode="C", 
+												verbose=FALSE){
     if(!reg)
         stop("This method does not work on sparse data!")
 
     ##evaluation
     ptm <- proc.time()
-    res <- fscm(data, k, reg=reg, regTime=regTime, funcyCtrlMbc,
-            location=location, scale=scale, knn=knn, useCode=useCode, verbose=verbose)
+    
+    res <- fscm(data, 
+    						k,
+    						reg=reg,
+    						regTime=regTime,
+    						funcyCtrlMbc,
+    						location=location,
+    						scale=scale, 
+    						knn=knn, 
+    						useCode=useCode, 
+    						verbose=verbose)
+    
     sysTime <- proc.time()-ptm
 
     ##funcyOut
@@ -307,8 +402,14 @@ fscmWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
 
 
 
-waveclustWrapper <- function(data, k, reg, regTime, funcyCtrlMbc,
-                             gamma="group", init="SEM", plotLoglik=FALSE){
+waveclustWrapper <- function(data, 
+														 k, 
+														 reg, 
+														 regTime, 
+														 funcyCtrlMbc,
+                             gamma="group", 
+														 init="SEM", 
+														 plotLoglik=FALSE){
     
                      
     if(!reg)
